from pydantic import BaseModel
import configparser
import boto3
import os
from offline_results.common.constants import RATING, ATTRIBUTE1, \
    CATEGORY, CATEGORY_ID, SUBCATEGORY, SUBCATEGORY_ID, ACTORS, \
    ACTOR_NAME, ACTOR_ID, DIRECTORS, DIRECTOR_NAME, TAGS, \
    DEFAULT_NAN, TAGS_ID

config = configparser.RawConfigParser()
# config.read("{}/{}".format(os.path.join(os.path.dirname(__file__)), "config.ini"))


class Config(BaseModel):
    aws_access_key_id: str = os.getenv(
        "AWS_ACCESS_KEY_ID", "AKIA2YVKI2GF5O6VSN7S")
    aws_secret_access_key: str = os.getenv(
        "AWS_SECRET_ACCESS_KEY", "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")
    region_name: str = os.getenv("AWS_REGION_NAME", "ap-southeast-1")
    s3_bucket_name: str = os.getenv(
        "AWS_BUCKET_NAME", "dev-visionplus")

S3_RESOURCE = boto3.resource(
    "s3",
    aws_access_key_id=Config().aws_access_key_id,
    aws_secret_access_key=Config().aws_secret_access_key,
    region_name=Config().region_name,
)

KMEANS_FEATURE = "kmeans"
MINIBATCH_KMEANS_FEATURE = "minibatch_kmeans"
WARDS_HIERARCHICAL_FEATURE = "wards_hierarchical"
KMEDOIDS_FEATURE = "kmedoids"
DBSCAN_FEATURE = "dbscan"
OPTICS_FEATURE = "optics"
BIRCH_FEATURE = "birch"
BIRCH_ENSEMBLE_FEATURE = "birch_ensemble"
S3_PAYTV_PREFIX = "paytv_"
S3_NONPAYTV_PREFIX = "nonpaytv_"
IS_PAYTV_PROPERTY_LABEL = "is_paytv"

CLUSTERING_METHODS = [
    KMEANS_FEATURE,
    MINIBATCH_KMEANS_FEATURE,
    WARDS_HIERARCHICAL_FEATURE,
    KMEDOIDS_FEATURE,
    DBSCAN_FEATURE,
    OPTICS_FEATURE,
    BIRCH_FEATURE,
    BIRCH_ENSEMBLE_FEATURE
]

TO_READ_FROM_S3_PREFERENCES = {
    RATING: [
        'HAS_RATING_PREFERENCE',
        RATING
    ],
    ATTRIBUTE1: [
        'HAS_ATTRIBUTE1_PREFERENCE',
        ATTRIBUTE1
    ],
    CATEGORY: [
        'HAS_CATEGORY_PREFERENCE',
        CATEGORY_ID
    ],
    SUBCATEGORY: [
        'HAS_SUBCATEGORY_PREFERENCE',
        SUBCATEGORY_ID
    ],
    ACTORS: [
        'HAS_ACTOR_PREFERENCE',
        ACTOR_NAME
    ],
    DIRECTORS: [
        'HAS_DIRECTOR_PREFERENCE',
        DIRECTOR_NAME
    ],
    TAGS: [
        'HAS_TAGS_PREFERENCE',
        TAGS
    ]
}

TO_READ_FROM_S3_CLUSTERING_LABELS = {
    KMEANS_FEATURE: [
        'HAS_KMEANS_CLUSTER',
        KMEANS_FEATURE
    ],

    MINIBATCH_KMEANS_FEATURE: [
        'HAS_MINIBATCH_CLUSTER',
        MINIBATCH_KMEANS_FEATURE
    ],

    WARDS_HIERARCHICAL_FEATURE: [
        'HAS_WARDS_CLUSTER',
        WARDS_HIERARCHICAL_FEATURE
    ],

    KMEDOIDS_FEATURE: [
        'HAS_KMEDOIDS_CLUSTER',
        KMEDOIDS_FEATURE
    ],

    DBSCAN_FEATURE: [
        'HAS_DBSCAN_CLUSTER',
        DBSCAN_FEATURE
    ],

    OPTICS_FEATURE: [
        'HAS_OPTICS_CLUSTER',
        OPTICS_FEATURE
    ],
    BIRCH_FEATURE: [
        'HAS_BIRCH_CLUSTER',
        BIRCH_FEATURE
    ],
    BIRCH_ENSEMBLE_FEATURE: [
        'HAS_ENSEMBLE_CLUSTER',
        BIRCH_ENSEMBLE_FEATURE
    ]
}

HAS_PAYTV_PROVIDER = "HAS_PAYTV_PROVIDER"

"""
The list of possible rating values that
can be assigned to content nodes in graphDB.
"""
RATING_VALUES = [
    "a",
    "bo",
    "bo-a",
    "bo-su",
    "d",
    "r",
    "su"
]

"""
To generate pre-trained BERT embeddings
"""
TOKENIZER = "bert-base-uncased"
MODEL = "bert-base-uncased"
ZERO_VECTOR_DIMENSION = 768

"""
List of preference attributes to discard
"""
DISCARD_ATTRIBUTES = [0, DEFAULT_NAN, '']

"""
UBD Data file to be read from S3
"""
FILENAME_VIDEO_MEASURE = "video_measure_data"

"""
homepage category recommendation 
data filtration configs
"""
OUTLIERS_CUSTOMER = ['0']
OUTLIERS_HOMEPAGE_ID = [306, 669]
DUPLICATE_VIEWS_CAP = 12
DEFAULT_HOMEPAGE_ID = -1
LOWER_FREQ_HOMEPAGE_ID_CUTOFF = 100

"""
Homepage content recommendations
"""
CONTENT_VIEW_LOWER_THRESHOLD = 10
CONTENT_VIEW_UPPER_THRESHOLD = 540

""" User preferences """
DURATION_CUTOFF = 110

PREF_FEATURES = {
    RATING: [
        'HAS_RATING_PREFERENCE',
        RATING
    ],
    ATTRIBUTE1: [
        'HAS_ATTRIBUTE1_PREFERENCE',
        ATTRIBUTE1
    ],
    CATEGORY: [
        'HAS_CATEGORY_PREFERENCE',
        CATEGORY_ID
    ],
    SUBCATEGORY: [
        'HAS_SUBCATEGORY_PREFERENCE',
        SUBCATEGORY_ID
    ],
    ACTORS: [
        'HAS_ACTOR_PREFERENCE',
        ACTOR_ID
    ],
    DIRECTORS: [
        'HAS_DIRECTOR_PREFERENCE',
        ACTOR_ID
    ],
    TAGS: [
        'HAS_TAGS_PREFERENCE',
        TAGS_ID
    ]
}

""" S3 """
BUCKET_NAME = "visionplus-dev"
USER_MAPPING_KEY = "data/historical/user/raw/mapping/"
"""
User-User filtering
"""
MINIMUM_SIMILAR_USERS_THRESHOLD = 5


#s3 bucket name
VISIONPLUS_DEV = "visionplus-dev"
DEV_VISIONPLUS = "dev-visionplus"

USER_PREFERENCES_PATH = "data/database/user_profile/processed_data_for_an/users_preferences.csv"
RAW_DATA_PATH = "data/historical/user/raw/mapping/31052022/"
UBD_DATA_PATH = "pickles/31052022/ubd.gzip"
HOMEPAGE_HAVING_CONTENT_PATH = 'mapping/24052022/Homepage_Having_Content.csv'
MODELS_PATH = "/models/"
STREAMING_VDB_KEY = 'vdb/2022/'
IMPLICIT_RATING_CSV = "mapping/24052022/implicit_rating.csv"
USER_CLUSTER_DIST = "data/database/user_profile/processed_data_for_an/user_cluster_dist.pkl"
PAY_TV_CENTROID_FEATURES = "data/historical/user/raw/mapping/31052022/Paytv_minibatch_kmeans_centroids_feature.csv"
NO_PAY_TV_CENTROID_FEATURES = "data/historical/user/raw/mapping/31052022/No_paytv_minibatch_kmeans_centroids_feature.csv"
PAY_TV_USER_CLUSTER_FEATURE = "data/historical/user/raw/mapping/31052022/Paytv_minibatch_kmeans_users_clustering_feature.csv"
NO_PAY_TV_USER_CLUSTER_FEATURE = 'data/historical/user/raw/mapping/31052022/No_paytv_minibatch_kmeans_users_clustering_feature.csv'

