from offline_results.common.constants import PAY_TV_CONTENT, STATUS, CONTENT_ID, MINIBATCH_KMEANS, HOMEPAGE_ID, \
    PAY_TV, CBF_MODULE_NAME, HOMEPAGE_ID_BASED, SERVICE_NAME, SCORE, CLUSTER_ID, CREATED_ON, USER_LABEL, RECORDS, \
    ALL_CONTENT_BASED, NO_PAY_TV, NO_PAY_TV_CONTENT
from offline_results.recommendation.utils import RecommendationUtils
from offline_results.recommendation.mean_user_from_cluster import MeanUserFromCluster
from offline_results.recommendation.homepage_recommendation.core.with_content_based_filtering.generate_recommendation import HomepageIdContentScorePred
from pandas import merge
from datetime import datetime
import logging

class HICRContentBasedFilteringController(
    RecommendationUtils,
    MeanUserFromCluster,
    HomepageIdContentScorePred
):

    def get_data(
            self,
            pay_tv_label,
    ):
        try:
            self.cluster_ids = self.find_all_cluster_id(pay_tv_label='True' if pay_tv_label is PAY_TV else 'False')
            self.users_dist = self.get_all_user_cluster_dist(pay_tv_label=PAY_TV if pay_tv_label is PAY_TV else NO_PAY_TV)
            content_with_homepage = self.homepage_data(pay_tv_label=PAY_TV_CONTENT if pay_tv_label is PAY_TV else NO_PAY_TV_CONTENT)
            self.content_with_homepage = content_with_homepage[[HOMEPAGE_ID, CONTENT_ID]]
            self.content_with_homepage[HOMEPAGE_ID] = content_with_homepage[HOMEPAGE_ID].apply(lambda x: x[0])
            active_homepages = list(self.active_homepage_ids())
            active_homepage_ids = self.content_with_homepage[self.content_with_homepage[HOMEPAGE_ID].isin(active_homepages)]
            self.active_homepage_id = set(active_homepage_ids[HOMEPAGE_ID].to_list())
        except Exception as e:
            logging.error(f"Error while preparing data for {pay_tv_label}, Error: {e}")

    def cbf_homepage_id_based_rec(
            self,
            pay_tv_label,
    ):
        try:
            homepage_id_based_rec = {}
            key_prefix = SERVICE_NAME + ":" + CBF_MODULE_NAME + ":" + pay_tv_label + ":" + HOMEPAGE_ID_BASED
            for cluster_id in self.cluster_ids[MINIBATCH_KMEANS]:
                key_prefix_cls = key_prefix + ":" + str(cluster_id)
                for homepage_id in list(self.active_homepage_id):
                    key_prefix_cls_hid = key_prefix_cls + ":" + str(homepage_id)
                    homepage_content = self.content_with_homepage[self.content_with_homepage[HOMEPAGE_ID] == homepage_id]
                    mean_user = self.get_mean_user(cluster_id=cluster_id, users_dist=self.users_dist)
                    result = self.get_score(user=mean_user, contents=homepage_content[CONTENT_ID].to_list())
                    result = merge(homepage_content, result, on=CONTENT_ID, how="left").reset_index(drop=True)
                    result[CLUSTER_ID] = cluster_id
                    result[CREATED_ON] = datetime.utcnow().isoformat()
                    result = result.drop(USER_LABEL, axis=1)
                    result = result.sort_values(SCORE, ascending=False, ignore_index=True)
                    result = result.drop_duplicates(ignore_index=True)
                    homepage_id_based_rec[key_prefix_cls_hid] = result.to_dict(RECORDS)
            return homepage_id_based_rec
        except Exception as e:
            logging.error(f"Error while preparing homepage id based recommendation for {pay_tv_label}, Error: {e}")

    def cbf_all_content_based_rec(
            self,
            pay_tv_label
    ):
        try:
            all_content_based_rec = {}
            key_prefix = SERVICE_NAME + ":" + CBF_MODULE_NAME + ":" + pay_tv_label + ":" + ALL_CONTENT_BASED
            for cluster_id in self.cluster_ids[MINIBATCH_KMEANS]:
                key_prefix_cls = key_prefix + ":" + str(cluster_id)
                mean_user = self.get_mean_user(cluster_id=cluster_id, users_dist=self.users_dist)
                result = self.get_score(user=mean_user, contents=self.content_with_homepage[CONTENT_ID].to_list())
                result = merge(self.content_with_homepage, result, on=CONTENT_ID, how="left").reset_index(drop=True)
                result[CLUSTER_ID] = cluster_id
                result[CREATED_ON] = datetime.utcnow().isoformat()
                result = result.drop(USER_LABEL, axis=1)
                result = result.groupby(CONTENT_ID).first().reset_index()
                result = result.sort_values(SCORE, ascending=False, ignore_index=True)
                all_content_based_rec[key_prefix_cls] = result.to_dict("records")
            return all_content_based_rec
        except Exception as e:
            logging.error(f"Error while preparing all content based rec for {pay_tv_label}, Error: {e}")

    def content_based_filtering_homepage_id_data(
            self
    ):
        #paytv homepage id content recommendation
        self.pay_tv_label = PAY_TV
        self.get_data(PAY_TV)
        pay_tv_homepage_id_based_rec = self.cbf_homepage_id_based_rec(PAY_TV)

        #no_paytv homepage id content recommendation
        self.pay_tv_label = NO_PAY_TV
        self.get_data(NO_PAY_TV)
        no_pay_tv_homepage_id_based_rec = self.cbf_homepage_id_based_rec(NO_PAY_TV)

        return pay_tv_homepage_id_based_rec, no_pay_tv_homepage_id_based_rec

    def content_based_filtering_all_content_data(
            self
    ):
        # paytv all_content based content recommendation
        self.pay_tv_label = PAY_TV
        self.get_data(PAY_TV)
        pay_tv_all_content_based_rec = self.cbf_all_content_based_rec(PAY_TV)

        # no_paytv all_content_based content recommendation
        self.pay_tv_label = NO_PAY_TV
        self.get_data(NO_PAY_TV)
        no_pay_tv_all_content_based_rec = self.cbf_all_content_based_rec(NO_PAY_TV)

        return pay_tv_all_content_based_rec, no_pay_tv_all_content_based_rec

