from offline_results.preferences.behaviour import PreprocessBehaviour
from offline_results.preferences.generate_preferences import PreferenceGenerator
from offline_results.common.constants import CUSTOMER_ID, \
    SOLO_FEATURE_LIST, FEATURE_DICT
from pandas import DataFrame, merge
from functools import reduce


class Preferences:
    def __init__(
            self,
            df: DataFrame
    ):
        """
        :param df: dataframe object pandas
        """
        self.df = df

    def controller(
            self,

    ) -> DataFrame:
        """
        Driver method for class MainImplementation which produces
        final_merged_df after complete preprocessing and
        preferences generation for user profile part.
        :return: preprocessed and user preference dataframe object pandas
        """
        appended_data = []

        behaviour = PreprocessBehaviour()

        solo_features = behaviour.controller(
            data=self.df,
            to_explode=False
        )
        for feature in SOLO_FEATURE_LIST:
            print("Generating User Preferences for the feature ---> ", feature)
            preference = PreferenceGenerator(
                feature=feature,
                feature_cutoff=2,
                user_cutoff=2
            )
            temp = preference.controller(
                data=solo_features,
            )

            appended_data.append(temp)

        for features, value in FEATURE_DICT.items():
            print("Generating User Preferences for the feature ---> ", features)
            pref = PreferenceGenerator(
                feature=features,
                feature_cutoff=2,
                user_cutoff=2)
            temp = behaviour.controller(
                data=self.df,
                to_explode=True,
                feature=features,
                key=value
            )
            temp = pref.controller(
                data=temp,
            )

            appended_data.append(temp)

        merged_df = reduce(
            lambda l, r: merge(l, r, on=CUSTOMER_ID, how='inner'),
            appended_data
        )

        return merged_df
