from pandas import DataFrame, merge
from offline_results.common.read_write_from_s3 import ConnectS3
from offline_results.common.config import FILENAME_VIDEO_MEASURE, VISIONPLUS_DEV, USER_PREFERENCES_PATH, \
    RAW_DATA_PATH, UBD_DATA_PATH, HOMEPAGE_HAVING_CONTENT_PATH
from offline_results.common.constants import CSV_EXTENSION, TAGS, SUBCATEGORY, CATEGORY, HOMEPAGE_HAVING_CONTENT, \
    HOME_PAGE_HAVING_CONTENT, CUSTOMER_ID, UBD, USER_PREFERENCES, S3, GRAPH_DB, HOMEPAGE_ID, TOD, DURATION, \
    GZIP_EXTENSION
import functools as ft
import logging
from pandas import read_csv
logging.basicConfig(level=logging.INFO)

class DataGathering:

    def __init__(
            self,
            data_source=S3,
    ):
        self.data_source= data_source
        self.ubd = DataFrame()
        self.homepage_having_content = DataFrame()
        self.category_pref = DataFrame()
        self.sub_category_pref = DataFrame()
        self.tags_pref = DataFrame()
        self.preferences = DataFrame()

    def save_preferences_to_s3(
            self,
            preferences
    ):

        ctl= ConnectS3()
        resource = ctl.create_connection()
        ctl.write_csv_to_s3(
            bucket_name=VISIONPLUS_DEV,
            object_name=USER_PREFERENCES_PATH,
            df_to_upload=preferences,
            resource=resource
        )

    def get_from_s3(
            self,
            attribute,
            object_path = RAW_DATA_PATH,
            bucket_name = VISIONPLUS_DEV,
    ):
        try:
            ctl = ConnectS3()
            resource = ctl.create_connection()
            result = ctl.read_compress_pickles_from_S3(
                bucket_name=bucket_name,
                object_name=object_path + attribute + "_preferences" + GZIP_EXTENSION,
                resource=resource
            )
            return result
        except Exception as e:
            print(e)

    def data_gathering(
            self
    ):
        if self.data_source == S3:
            self.category_pref = self.get_from_s3(CATEGORY)
            self.sub_category_pref = self.get_from_s3(SUBCATEGORY)
            self.tags_pref = self.get_from_s3(TAGS)
            self.tod_pref = self.get_from_s3(TOD)
            self.duration_pref = self.get_from_s3(DURATION)

            ctl = ConnectS3()
            resource = ctl.create_connection()
            self.homepage_having_content = ctl.read_csv_from_s3(VISIONPLUS_DEV, HOMEPAGE_HAVING_CONTENT_PATH, resource)
            self.ubd = ctl.read_compress_pickles_from_S3(VISIONPLUS_DEV, UBD_DATA_PATH, resource)

            preferences = [self.category_pref, self.sub_category_pref, self.tags_pref, self.tod_pref, self.duration_pref]
            self.preferences = ft.reduce(lambda left, right: merge(left, right, on=CUSTOMER_ID), preferences)
            self.preferences[CUSTOMER_ID] = self.preferences[CUSTOMER_ID].astype(int)
            self.ubd[CUSTOMER_ID] = self.ubd[CUSTOMER_ID].astype(int)

            self.save_preferences_to_s3(self.preferences)
            self.data = {
                UBD: self.ubd,
                USER_PREFERENCES: self.preferences,
                HOMEPAGE_HAVING_CONTENT: self.homepage_having_content
            }

        if self.data_source == GRAPH_DB:
            """update code here"""
            pass

##test code here
# if __name__ =="__main__":
#     ctl= DataGathering()
#     ctl.data_gathering()