from pandas import DataFrame, merge
from collections import Counter

from offline_results.common.config import VISIONPLUS_DEV
from offline_results.common.read_write_from_s3 import ConnectS3
from offline_results.common.constants import HOMEPAGE_VALID_INVALID_IDS, CUSTOMER_ID, COUNT, CONTENT_ID, VIDEO_ID1, \
    HOMEPAGE_ID, UBD, HOMEPAGE_HAVING_CONTENT, USER_PREFERENCES, CONTENT_ID_CONTENT, VALID_IDS, INVALID_IDS, ONE_HOT_ENCODER
from offline_results.recommendation.utils import RecommendationUtils
from offline_results.recommendation.homepage_cluster_category.config import HOMEPAGE_ID_VIEW_COUNT_THRESHOLD, \
    BIASED_HOMEPAGE_IDS, BIASED_CUSTOMER_ID, NUMBER_OF_DUPLICATE_VIEWS_THRESHOLD
import logging
logging.basicConfig(level=logging.INFO)


class DataPreprocessing():

    def data_cleaning(
            self,
            vm_data
    ):
        vm_data = vm_data[[CUSTOMER_ID, VIDEO_ID1]]
        vm_data = vm_data[vm_data[CUSTOMER_ID] != '\\N']
        vm_data = vm_data[vm_data[VIDEO_ID1] != '\\N']
        vm_data = vm_data[vm_data[VIDEO_ID1] != '0,0']
        vm_data[VIDEO_ID1] = vm_data[VIDEO_ID1].apply(str)
        vm_data[VIDEO_ID1] = vm_data[VIDEO_ID1].apply(lambda x: x.replace(",", ""))
        vm_data = vm_data[vm_data[VIDEO_ID1] != "nan"]
        vm_data[VIDEO_ID1] = vm_data[VIDEO_ID1].apply(int)

        return vm_data

    def filter_duplicate_views(self, ubd_df) -> DataFrame:
        duplicate_views_df = ubd_df.groupby([CUSTOMER_ID, VIDEO_ID1]).size()
        duplicate_views_df = DataFrame(duplicate_views_df, columns=[COUNT])
        duplicate_views_df = duplicate_views_df[duplicate_views_df[COUNT] < NUMBER_OF_DUPLICATE_VIEWS_THRESHOLD]
        for idx, val in duplicate_views_df.iterrows():
            if val[COUNT] <= NUMBER_OF_DUPLICATE_VIEWS_THRESHOLD:
                val[COUNT] = val[COUNT]
            else:
                val[COUNT] = NUMBER_OF_DUPLICATE_VIEWS_THRESHOLD
        filtered_df = merge(duplicate_views_df, ubd_df, on=[CUSTOMER_ID, VIDEO_ID1])
        filtered_df = filtered_df[filtered_df.groupby([CUSTOMER_ID, VIDEO_ID1]).customer_id.transform(
            COUNT) <= NUMBER_OF_DUPLICATE_VIEWS_THRESHOLD]
        final_df = filtered_df.drop(COUNT, axis=1)
        return final_df

    def data_filtering(
            self,
            vm_data,
            df_home,
    ):

        vm_data = vm_data.merge(df_home[[HOMEPAGE_ID, CONTENT_ID]], left_on=VIDEO_ID1, right_on=CONTENT_ID, how="left")
        vm_data = vm_data[(~vm_data[HOMEPAGE_ID].isin(BIASED_HOMEPAGE_IDS)) & (vm_data[CUSTOMER_ID] != BIASED_CUSTOMER_ID)]
        vm_data = self.filter_duplicate_views(vm_data)
        vm_data = vm_data[~vm_data[HOMEPAGE_ID].isna()]
        vm_data[HOMEPAGE_ID] = vm_data[HOMEPAGE_ID].apply(int)
        vm_data[CONTENT_ID] = vm_data[CONTENT_ID].apply(int)
        return vm_data

    def save_homepage_ids(self, homepage_ids):
        ctl = ConnectS3()
        resource = ctl.create_connection()
        ctl.write_pkl_to_s3(
            bucket_name=VISIONPLUS_DEV,
            object_name=HOMEPAGE_VALID_INVALID_IDS,
            data=homepage_ids,
            resource=resource
        )

    def data_capping(
            self,
            vm_data,
            df_home
    ):
        ids = dict(Counter(vm_data[HOMEPAGE_ID]))
        valid_ids = [id for id in ids if ids[id] > HOMEPAGE_ID_VIEW_COUNT_THRESHOLD]
        invalid_ids = [id for id in ids if ids[id] <= HOMEPAGE_ID_VIEW_COUNT_THRESHOLD]
        vm_data[HOMEPAGE_ID] = vm_data[HOMEPAGE_ID].replace(invalid_ids, -1)
        vm = DataFrame(vm_data.groupby([CUSTOMER_ID])[HOMEPAGE_ID].apply(set)).reset_index()
        vm[HOMEPAGE_ID] = vm[HOMEPAGE_ID].apply(list)
        vm[CUSTOMER_ID] = vm[CUSTOMER_ID].apply(int)
        unique_ids = list(set(df_home[HOMEPAGE_ID]))
        one_hot_encoder= list(set([id if id not in invalid_ids else -1 for id in unique_ids]))
        one_hot_encoder.sort()
        homepage_ids_info = {VALID_IDS: valid_ids, INVALID_IDS: invalid_ids, ONE_HOT_ENCODER: one_hot_encoder}
        self.save_homepage_ids(homepage_ids_info)
        return vm, homepage_ids_info

    def merge_label_and_features(
            self,
            vm_data,
            user_preferences
    ):
        dataset = vm_data[[CUSTOMER_ID, HOMEPAGE_ID]].merge(user_preferences, on=CUSTOMER_ID)
        dataset = dataset.drop(columns=[CUSTOMER_ID])
        return dataset

    def label_encoding(
            self,
            dataset,
            one_hot_encoder
    ):

        ctl=RecommendationUtils()
        dataset[HOMEPAGE_ID] = dataset[HOMEPAGE_ID].apply(lambda x: ctl.multi_label_encoding(x, one_hot_encoder))
        return dataset

    def data_preprocessing(
            self,
            data
    ):
        vm_data = data[UBD]
        df_home = data[HOMEPAGE_HAVING_CONTENT]
        user_preferences = data[USER_PREFERENCES]
        vm_data = self.data_cleaning(vm_data)
        vm_data = self.data_filtering(vm_data, df_home)
        vm_data, homepage_ids_info = self.data_capping(vm_data, df_home)
        dataset = self.merge_label_and_features(vm_data, user_preferences)
        self.dataset = self.label_encoding(dataset, homepage_ids_info[ONE_HOT_ENCODER])









