import logging
import os
import pickle
import random
from pandas import DataFrame
from offline_results.common.config import MODELS_PATH, VISIONPLUS_DEV, USER_PREFERENCES_PATH
from offline_results.common.constants import HOMEPAGE_VALID_INVALID_IDS, INVALID_IDS, ONE_HOT_ENCODER,\
    CUSTOMER_ID, HOMEPAGE_ID, SCORE, CURRENT_MODEL_NAME
from offline_results.recommendation.homepage_cluster_category.config import HOMEPAGE_ID_CLF_MODEL
from offline_results.common.read_write_from_s3 import ConnectS3
import logging
logging.basicConfig(level=logging.INFO)

class HomepageIdRecommendation:

    @staticmethod
    def load_model_from_s3(model_name=HOMEPAGE_ID_CLF_MODEL[CURRENT_MODEL_NAME]):
        try:
            path = os.getcwd() + "\\" + model_name
            if os.path.isfile(path):
                model = pickle.load(open(path, 'rb'))
                return model
            ctl = ConnectS3()
            resource = ctl.create_connection()
            ctl.download_from_s3(
                bucket_name=VISIONPLUS_DEV,
                filename_with_path=path,
                key=MODELS_PATH + HOMEPAGE_ID + "/" + model_name,
                resource=resource
            )
            model = pickle.load(open(path, 'rb'))
            return model
        except Exception as e:
            logging.error(f"Error while uploading model {model_name} on s3, Error: {e}")

    @staticmethod
    def get_user_preferences(
            user_id,
    ):
        ctl = ConnectS3()
        resource = ctl.create_connection()
        df = ctl.read_csv_from_s3(
            bucket_name=VISIONPLUS_DEV,
            object_name=USER_PREFERENCES_PATH,
            resource=resource
        )
        user_pre = df[df[CUSTOMER_ID] == user_id].drop(columns=[CUSTOMER_ID])
        return user_pre.to_numpy()


    @staticmethod
    def get_hompage_ids_info():
        ctl = ConnectS3()
        resource = ctl.create_connection()
        homepage_ids_info=ctl.read_pkl_from_S3(
            bucket_name=VISIONPLUS_DEV,
            object_name=HOMEPAGE_VALID_INVALID_IDS,
            resource=resource
        )
        return homepage_ids_info

    def generate_recommendations(
            self,
            model,
            user_id
    ):
        user_preferences = self.get_user_preferences(user_id)
        yhat = model.predict(user_preferences)[0]
        homepage_ids_info = self.get_hompage_ids_info()
        invalide_ids = homepage_ids_info[INVALID_IDS]
        one_hot_encoder = homepage_ids_info[ONE_HOT_ENCODER]
        recomm = DataFrame({HOMEPAGE_ID: one_hot_encoder, SCORE: yhat})
        recomm = recomm.replace(-1, invalide_ids[random.randint(0, len(invalide_ids)-1)])
        recomm = recomm.sort_values([SCORE], ascending=False).reset_index(drop=True)
        return recomm

# #test code here
# if __name__=="__main__":
#     ctl=HomepageIdRecommendation()
#     model = ctl.load_model_from_s3()
#     recomm=ctl.generate_recommendations(model=model, user_id=0)
#     print(recomm)