from typing import ClassVar

import ast
from operator import itemgetter
from datetime import datetime
import pandas as pd
from graphdb import GraphDb, GraphDbConnection
from pandas import DataFrame

from offline_results.similarity.content_profile.similarity_all_contents import SimilarityAllContents
from offline_results.common.constants import USER_LABEL, USER_CONTENT_RELATIONSHIP_LABEL, CUSTOMER_ID, CONTENT_ID, \
    CREATED_ON, VIEW_HISTORY, VIEW_COUNT, RECENT_VIEWED_DATE, DURATION


def get_recent_viewed_date_and_duration(history):
    history_data = history[1][VIEW_HISTORY]
    list_history_data = ast.literal_eval(history_data)
    list_history_data = sorted(list_history_data, key=itemgetter(CREATED_ON), reverse=True)
    recent_date = list_history_data[0][CREATED_ON]
    recent_date = datetime.strptime(recent_date, '%Y/%m/%d').date()

    total_duration = 0
    for history in list_history_data:
        duration_content = history[DURATION]
        total_duration += duration_content
    return recent_date, total_duration


class BYWData:

    def __init__(
            self,
            connection_class: GraphDbConnection
    ):
        self.graph = GraphDb.from_connection(connection_class)

    @classmethod
    def from_connection_uri(
            cls,
            connection_uri: str
    ) -> ClassVar:
        """Create new object based on connection uri
        :param connection_uri: string connection uri
        :return: object class
        """
        return cls(GraphDbConnection.from_uri(connection_uri))

    @classmethod
    def from_connection_class(
            cls,
            connection_class: GraphDbConnection
    ) -> ClassVar:
        """Define new class based on object connection
        :param connection_class: object connection class
        :return: object class
        """
        return cls(connection_class)

    @staticmethod
    def get_all_content_similarity(content_label, graph):

        return SimilarityAllContents.prepare_similarity_based_on_all_content(content_label, graph)

    @staticmethod
    def get_viewed_content(graph):
        views_query = graph.custom_query(f'''
        g.V().hasLabel('{USER_LABEL}').outE().hasLabel('{USER_CONTENT_RELATIONSHIP_LABEL}').inV().path().by(elementMap())
        ''', payload={
            USER_LABEL: USER_LABEL,
            USER_CONTENT_RELATIONSHIP_LABEL: USER_CONTENT_RELATIONSHIP_LABEL
        })
        user_content_df = DataFrame()
        for view in views_query:
            for history in view:
                user_object_data = history[0][CUSTOMER_ID]
                content_object_data = history[2][CONTENT_ID]
                recent_viewed_date_data, total_duration = get_recent_viewed_date_and_duration(history)
                view_count_data = history[1][VIEW_COUNT]
                user_content_data = pd.DataFrame([{CUSTOMER_ID: user_object_data,
                                                   CONTENT_ID: content_object_data,
                                                   RECENT_VIEWED_DATE: recent_viewed_date_data,
                                                   VIEW_COUNT: view_count_data,
                                                   DURATION: total_duration}])
                user_content_df = pd.concat([user_content_df, user_content_data], axis=0)
                user_content_df = user_content_df.sort_values(by=[RECENT_VIEWED_DATE, DURATION, VIEW_COUNT], ascending=False)

        user_content_df_grouped = pd.DataFrame(
            user_content_df.groupby(CUSTOMER_ID)[CONTENT_ID].apply(list).reset_index()
        )

        return user_content_df_grouped
