from typing import ClassVar

from graphdb import GraphDb, GraphDbConnection

from offline_results.common.constants import PAY_TV_CONTENT
from offline_results.recommendation.homepage_recommendation.core.with_cf_item_to_item.utils import ItemToItemUtils


class CF_II:
    def __init__(
            self,
            connection_class: GraphDbConnection
    ):
        self.graph = GraphDb.from_connection(connection_class)

    @classmethod
    def from_connection_uri(
            cls,
            connection_uri_writer: str,
            connection_uri_reader: str,
            pool_size: int = 0,
            max_workers: int = 0,
    ) -> ClassVar:
        """Create new object based on connection uri
        :param connection_uri: string connection uri
        :return: object class
        """
        return cls(GraphDbConnection.from_uri(connection_uri_writer, connection_uri_reader, pool_size, max_workers))

    @classmethod
    def from_connection_class(
            cls,
            connection_class: GraphDbConnection
    ) -> ClassVar:
        """Define new class based on object connection
        :param connection_class: object connection class
        :return: object class
        """
        return cls(connection_class)

    def get_contents_with_highest_view_count(self):
        data_paytv, data_no_paytv = ItemToItemUtils.user_content_network_to_dataframe(self.graph)

        highest_view_count_data_paytv, highest_view_count_data_no_paytv = ItemToItemUtils.find_maximum_view_count(
            data_paytv), ItemToItemUtils.find_maximum_view_count(data_no_paytv)

        return highest_view_count_data_paytv, highest_view_count_data_no_paytv

    def homepage_id_mapping(self):
        highest_view_count_data_paytv, highest_view_count_data_no_paytv = self.get_contents_with_highest_view_count()

        data_homepage_paytv, data_homepage_no_paytv = ItemToItemUtils.homepage_network_to_dataframe_with_content_paytv_flag(
            self.graph)

        homepage_paytv_merged_with_data, homepage_no_paytv_merged_with_data = ItemToItemUtils.merge_on_content_id(
            highest_view_count_data_paytv, data_homepage_paytv), ItemToItemUtils.merge_on_content_id(
            highest_view_count_data_no_paytv, data_homepage_no_paytv)

        return homepage_paytv_merged_with_data, homepage_no_paytv_merged_with_data

    def get_most_similar_top_ten_contents_based_on_all(self):
        homepage_paytv_merged_with_data, homepage_no_paytv_merged_with_data = self.homepage_id_mapping()

        paytv_df, no_paytv_df = ItemToItemUtils.get_similar_contents_based_on_all(self.graph)

        similarity_df_paytv, similarity_df_no_paytv = ItemToItemUtils.merge_on_content_id(
            homepage_paytv_merged_with_data, paytv_df), ItemToItemUtils.merge_on_content_id(
            homepage_no_paytv_merged_with_data, no_paytv_df)

        return ItemToItemUtils.add_time_created_attribute(
            similarity_df_paytv), ItemToItemUtils.add_time_created_attribute(similarity_df_no_paytv)

    def get_most_similar_top_ten_contents_based_on_homepage_id(self, customer_id, content_label):
        pay_tv_df, no_pay_tv_df = self.homepage_id_mapping()

        status_mapped_paytv_df, status_mapped_no_paytv_df = ItemToItemUtils.map_homepage_and_content_status(self.graph,
                                                                                                            pay_tv_df,
                                                                                                            no_pay_tv_df)

        if content_label == PAY_TV_CONTENT:
            recommendation_df = ItemToItemUtils.prepare_user_wise_recommendation_results_based_on_homepage_id(
                customer_id, content_label, self.graph, status_mapped_paytv_df)
        else:
            recommendation_df = ItemToItemUtils.prepare_user_wise_recommendation_results_based_on_homepage_id(
                customer_id, content_label, self.graph, status_mapped_no_paytv_df)

        return recommendation_df
