from traceback import print_exc

from offline_results.common.constants import USER_LABEL, USER_CONTENT_RELATIONSHIP_LABEL, PAY_TV_CONTENT, \
    NO_PAY_TV_CONTENT, HOMEPAGE


class QueryUtils:

    @staticmethod
    def query_log_data_for_all_user(graph):
        try:
            user_content_network = graph.custom_query(f'''
                    g.V().hasLabel('{USER_LABEL}').outE().hasLabel('{USER_CONTENT_RELATIONSHIP_LABEL}').inV().path().by(elementMap())
                    ''', payload={
                USER_LABEL: USER_LABEL,
                USER_CONTENT_RELATIONSHIP_LABEL: USER_CONTENT_RELATIONSHIP_LABEL
            })
        except Exception:
            print_exc()
            user_content_network = []

        return user_content_network

    @staticmethod
    def query_log_data_for_user_including_paytv_status(graph):
        try:
            response = graph.custom_query(
                query=f'''g.V().hasLabel('{USER_LABEL}')
                .outE('{USER_CONTENT_RELATIONSHIP_LABEL}').inV().
                hasLabel('{PAY_TV_CONTENT}').
                path().by(elementMap())''',
                payload={
                    USER_LABEL: USER_LABEL,
                    USER_CONTENT_RELATIONSHIP_LABEL: USER_CONTENT_RELATIONSHIP_LABEL,
                    PAY_TV_CONTENT: PAY_TV_CONTENT
                }
            )
        except Exception:
            print_exc()
            response = []
        return response

    @staticmethod
    def query_log_data_for_user_including_no_paytv_status(graph):
        try:
            response = graph.custom_query(
                query=f'''g.V().hasLabel('{USER_LABEL}')
                    .outE('{USER_CONTENT_RELATIONSHIP_LABEL}').inV().
                    hasLabel('{NO_PAY_TV_CONTENT}').
                    path().by(elementMap())''',
                payload={
                    USER_LABEL: USER_LABEL,
                    USER_CONTENT_RELATIONSHIP_LABEL: USER_CONTENT_RELATIONSHIP_LABEL,
                    PAY_TV_CONTENT: PAY_TV_CONTENT
                }
            )
        except Exception:
            print_exc()
            response = []
        return response

    @staticmethod
    def query_homepage_data_for_paytv_contents(graph):
        try:
            response = graph.custom_query(
                query=f'''g.V().hasLabel('{PAY_TV_CONTENT}').match(
                            __.as("c").values("content_id").as("content_id"),
                            __.as("c").out("HAS_HOMEPAGE")
                            .values("homepage_id").as("homepage_id")
                            ).select("content_id", "homepage_id")''',
                payload={
                    PAY_TV_CONTENT: PAY_TV_CONTENT
                })
        except Exception:
            print_exc()
            response = []

        return response

    @staticmethod
    def query_homepage_data_for_no_paytv_contents(graph):
        try:
            response = graph.custom_query(
                query=f'''g.V().hasLabel('{NO_PAY_TV_CONTENT}').match(
                            __.as("c").values("content_id").as("content_id"),
                            __.as("c").out("HAS_HOMEPAGE")
                            .values("homepage_id").as("homepage_id")
                            ).select("content_id", "homepage_id")''',
                payload={
                    NO_PAY_TV_CONTENT: NO_PAY_TV_CONTENT
                })
        except Exception:
            print_exc()
            response = []

        return response

    @staticmethod
    def query_static_homepage_data_for_homepage_status(graph):
        try:
            response = graph.custom_query(
                query=f'''g.V().hasLabel('{HOMEPAGE}').match(
                                    __.as("c").values("homepage_id").as("homepage_id"),
                                    __.as("c").values("homepage_status").as("homepage_status"),
                                    __.as("c").values("is_connected").as("is_connected")
                                    ).select("homepage_id", "homepage_status", "is_connected")''',
                payload={
                    HOMEPAGE: HOMEPAGE
                })
        except Exception:
            print_exc()
            response = []

        return response

    @staticmethod
    def query_pay_tv_content_data_for_content_status(graph):
        try:
            response = graph.custom_query(
                query=f'''g.V().hasLabel("{PAY_TV_CONTENT}").match
                            (
                            __.as("c").values("content_id").as("content_id"),
                            __.as("c").values("status").as("status")
                            )
                            .select("content_id", "status")''',
                payload={
                    PAY_TV_CONTENT: PAY_TV_CONTENT
                })
        except Exception:
            print_exc()
            response = []

        return response

    @staticmethod
    def query_no_pay_tv_content_data_for_content_status(graph):
        try:
            response = graph.custom_query(
                query=f'''g.V().hasLabel("{NO_PAY_TV_CONTENT}").match
                            (
                            __.as("c").values("content_id").as("content_id"),
                            __.as("c").values("status").as("status")
                            )
                            .select("content_id", "status")''',
                payload={
                    NO_PAY_TV_CONTENT: NO_PAY_TV_CONTENT
                }
            )
        except Exception:
            print_exc()
            response = []

        return response
