import os
import pickle
from pandas import DataFrame
from offline_results.common.config import MODELS_PATH, VISIONPLUS_DEV
from offline_results.common.constants import HOMEPAGE_ID, CURRENT_MODEL_NAME, USER_LABEL, CONTENT_ID, SCORE
from offline_results.recommendation.homepage_cluster_category.config import HOMEPAGE_IDS_CONTENT_REC_MODEL
from offline_results.common.read_write_from_s3 import ConnectS3
import logging
logging.basicConfig(level=logging.INFO)

class HomepageIdContentScorePred:

    @staticmethod
    def load_model_from_s3(model_name=HOMEPAGE_IDS_CONTENT_REC_MODEL[CURRENT_MODEL_NAME]):
        try:
            path = os.getcwd() + "\\" + model_name
            if os.path.isfile(path):
                model = pickle.load(open(path, 'rb'))
                return model
            ctl = ConnectS3()
            resource = ctl.create_connection()
            ctl.download_from_s3(
                bucket_name=VISIONPLUS_DEV,
                filename_with_path=path,
                key=MODELS_PATH + HOMEPAGE_ID + "/" + model_name,
                resource=resource
            )
            model = pickle.load(open(path, 'rb'))
            return model
        except Exception as e:
            logging.error(f"Error while uploading model {model_name} on s3, Error: {e}")

    def get_score(
            self,
            user,
            contents,
            model=None
    ):
        try:
            rec={USER_LABEL: [], CONTENT_ID: [], SCORE: []}
            loaded_model = self.load_model_from_s3() if model is None else model
            for content in contents:
                prediction = loaded_model.predict(uid=str(user), iid=int(content), verbose=True)
                score = prediction.est
                rec[USER_LABEL].append(user)
                rec[CONTENT_ID].append(content)
                rec[SCORE].append(score)
            rec_df = DataFrame(rec)
            rec_df = rec_df.sort_values([SCORE]).reset_index(drop=True)
            return rec_df

        except Exception as e:
            logging.error(f"Error while getting score. Error :{e}")

# #test code here
# if __name__ == "__main__":
#     ctl=HomepageIdContentRecommendation()
#     model = ctl.load_model_from_s3()
#     recomm = ctl.get_score(user=4120, contents=[15, 50, 11, 17173, 34410], model=model)
#     print(recomm)