import logging
from offline_results.common.constants import IS_PAY_TV, USER_LABEL, HOMEPAGE_ID, CONTENT_ID, \
    STATUS, PAY_TV_CONTENT, NO_PAY_TV_CONTENT, MINIBATCH_KMEANS, VIEWED
from offline_results.repository.graph_db_connection import ANGraphDb
from pandas import DataFrame
logging.basicConfig(level=logging.INFO)

class RecommendationUtils:

    @staticmethod
    def get_attribute_map(
            graph,
            node_label: str,
            key_node_property: str,
            value_node_property: str
    ) -> list:

        return graph.custom_query(
            query=f"""g.V().hasLabel('{node_label}').match(
            __.as("c").values('{key_node_property}').as('{key_node_property}'),
            __.as("c").values('{value_node_property}').as('{value_node_property}')
            ).select('{key_node_property}', '{value_node_property}')""",
            payload=
            {
                node_label: node_label,
                key_node_property: key_node_property,
                value_node_property: value_node_property
            }
        )

    @staticmethod
    def get_homepage_for_contents(
            graph
    ) -> list:

        response = []
        for label in [PAY_TV_CONTENT, NO_PAY_TV_CONTENT]:

            content_response = graph.custom_query(
                query=f'''g.V().hasLabel('{label}').match(
                __.as("c").values("content_id").as("content_id"),
                __.as("c").out("HAS_HOMEPAGE")
                .values("homepage_id").as("homepage_id")
                ).select("content_id", "homepage_id")''',
                payload={
                    label: label,
                })

            # appending the flattened custom query response
            response.extend([item
                             for subresponse in content_response
                             for item in subresponse
                             ])

        return response

    @staticmethod
    def get_homepage_id_title_map(
            graph,
            get_status: bool = False
    ) -> list:

        if not get_status:
            # TO DO : Add payload (with latest version of graphdb-module, it throws an error without payload)
            homepage_response = graph.custom_query(
                query=f'''g.V().hasLabel('homepage').match(
                    __.as("c").values("homepage_id").as("homepage_id"),
                    __.as("c").values("homepage_title_en").as("homepage_title_en"),
                    ).select("homepage_id", "homepage_title_en")''',
            )
        else:
            # TO DO : Add payload (with latest version of graphdb-module, it throws an error without payload)
            homepage_response = graph.custom_query(
                query=f'''g.V().hasLabel('homepage').match(
                                __.as("c").values("homepage_id").as("homepage_id"),
                                __.as("c").values("homepage_title_en").as("homepage_title_en"),
                                __.as("c").values("homepage_status").as("homepage_status"),
                                __.as("c").values("is_connected").as("is_connected")
                                ).select("homepage_id", "homepage_title_en", "homepage_status", "is_connected")''',
            )

        return [item
                for subresponse in homepage_response
                for item in subresponse]

    @staticmethod
    def get_node_property_label_values(
            graph,
            label: str,
            property: str
    ) -> list:
        # TO DO : Add payload (with latest version of graphdb-module, it throws an error without payload)
        response = graph.custom_query(
            query=f'''g.V().hasLabel('{label}').match(
                __.as("c").values('{property}').as('{property}')
                ).select('{property}')''',
        )

        return [item
                for subresponse in response
                for item in subresponse]

    @staticmethod
    def get_viewed_features(
            graph
    ) -> DataFrame:
        """
        Query graphdb for user_viewed_content relation.
        """

        viewed_feature_list = []
        # TO DO : Add payload (with latest version of graphdb-module, it throws an error without payload)
        response = graph.custom_query(
            query=f'''g.V().hasLabel('{USER_LABEL}').outE('{VIEWED}').inV().path().by(elementMap())''')

        for index, ele in enumerate(response):
            for i in range(len(response[index])):
                customer_id = response[index][i][0]['customer_id']
                view_count = response[index][i][1]['view_count']
                view_history = response[index][i][1]['view_history']
                content_id = response[index][i][2]['content_id']
                view_dict = {'customer_id': customer_id,
                             'content_id': content_id,
                             'view_count': view_count,
                             'view_history': view_history,
                             }
                viewed_feature_list.append(view_dict)

        return DataFrame(viewed_feature_list).reset_index(drop=True)

    @staticmethod
    def multi_label_encoding(arr, unique_ids):
        encoded_list, count = [0] * len(unique_ids), 0
        for i in unique_ids:
            if i in arr: encoded_list[count] = 1
            count += 1
        return encoded_list

    def find_with_has_label(
            self,
            label: list,
            edge_label: str,
            features: list
    ):
        feature_str = ""
        for feature in features:
            feature_str += "__.as('c').values('" + feature + "').as('" + feature + "'),"
        query = f"""
                g.V().hasLabel('{"','".join(label)}').outE('{edge_label}').match(
                {feature_str[:-1]}).select('{"','".join(features)}')
                """
        graph = ANGraphDb().graph
        data = graph.custom_query(query, payload={})
        data = [rec for idx in data for rec in idx]
        data = DataFrame(data)
        return data

    def find_with_has_conditon(
            self,
            label: str,
            condition_on,
            condition_value,
            edge_label,
            features: list
    ):

        feature_str = ""
        condition_value = f"{condition_value}" if condition_value.__class__ == str else condition_value
        for feature in features:
            feature_str += "__.as('c').values('" + feature + "').as('" + feature + "'),"
        query = f"""
                g.V().has('{label}','{condition_on}',{condition_value}).
                outE('{edge_label}').
                match({feature_str[:-1]}).select('{"','".join(features)}')
                """
        graph = ANGraphDb().graph
        logging.info(f"start fetching data from Graph Db for label {label}")
        data = graph.custom_query(query, payload={})
        data = [rec for idx in data for rec in idx]
        data = DataFrame(data)
        return data


    @staticmethod
    def homepage_data(
            pay_tv_label=NO_PAY_TV_CONTENT,
            features=[CONTENT_ID, HOMEPAGE_ID]
    ):
        feature_str = ""
        for feature in features:
            feature_str += "values('" + feature + "'),"
        query=f"""
            g.V().hasLabel('{pay_tv_label}','{STATUS}','Active').
            outE('HAS_HOMEPAGE').
            project('HAS_HOMEPAGE', '{CONTENT_ID}', 'homepage_id').
            by(label).
            by(outV().values('{CONTENT_ID}')).
            by(inV().union(
            {feature_str[:-1]}).fold())
            """
        graph = ANGraphDb().graph
        data = graph.custom_query(query, payload={HOMEPAGE_ID: HOMEPAGE_ID})
        data = [rec for idx in data for rec in idx]
        df = DataFrame(data)
        return df

    def find_all_cluster_id(
            self,
            pay_tv_label
    ):
        query=f"""
            g.V().has('{MINIBATCH_KMEANS}','{IS_PAY_TV}', '{pay_tv_label}' ).valueMap().by(unfold()).toList()
            """
        graph = ANGraphDb().graph
        data = graph.custom_query(query, payload={HOMEPAGE_ID: HOMEPAGE_ID})
        data = [rec for idx in data for rec in idx]
        df = DataFrame(data)
        return df

    def active_homepage_ids(
            self
    ):
        query = f"""
                g.V().has('homepage', 'homepage_status', 'Active').valueMap().by(unfold()).toList()
                """
        graph = ANGraphDb().graph
        data = graph.custom_query(query, payload={HOMEPAGE_ID: HOMEPAGE_ID})
        data = [rec for idx in data for rec in idx]
        df = DataFrame(data)[HOMEPAGE_ID]
        return df