from typing import ClassVar

from graphdb import GraphDb, GraphDbConnection

from offline_results.common.constants import CONTENT_CORE_SYNOPSIS
from offline_results.similarity.content_profile.utils.query_utils import QueryUtils
from offline_results.similarity.content_profile.utils.similarity_utils import SimilarityUtils


class SimilarityAllContents:
    def __init__(
            self,
            connection_class: GraphDbConnection
    ):
        self.graph = GraphDb.from_connection(connection_class)

    @classmethod
    def from_connection_uri(
            cls,
            connection_uri: str
    ) -> ClassVar:
        """Create new object based on connection uri
        :param connection_uri: string connection uri
        :return: object class
        """
        return cls(GraphDbConnection.from_uri(connection_uri, connection_uri))

    @classmethod
    def from_connection_class(
            cls,
            connection_class: GraphDbConnection
    ) -> ClassVar:
        """Define new class based on object connection
        :param connection_class: object connection class
        :return: object class
        """
        return cls(connection_class)

    @staticmethod
    def prepare_similarity_based_on_all_content(content_label, graph):
        all_content_dict_cos_sim = None
        try:
            all_content_cluster = QueryUtils.get_all_content(content_label, graph)

            all_content_df = SimilarityUtils.cluster_data_to_single_df(all_content_cluster)

            all_content_core_synopsys = QueryUtils.get_all_synopsys(CONTENT_CORE_SYNOPSIS, graph)

            all_content_new_df = SimilarityUtils.combine_features(all_content_df, graph, content_label,
                                                                  all_content_core_synopsys, all_content=True)

            all_content_tfidf = SimilarityUtils.create_tfidf_df(all_content_new_df)

            all_content_dict_cos_sim = SimilarityUtils.calculate_single_cosine_similarity(all_content_tfidf)
        except Exception as e:
            print(e)
            print('Not able to add content similarity based on home page id')
        return all_content_dict_cos_sim