from typing import ClassVar

from graphdb import GraphDb, GraphDbConnection

from offline_results.similarity.content_profile.utils.query_utils import QueryUtils
from offline_results.similarity.content_profile.utils.similarity_utils import SimilarityUtils


class SimilarityHomepageContents:

    def __init__(
            self,
            connection_class: GraphDbConnection
    ):
        self.graph = GraphDb.from_connection(connection_class)

    @classmethod
    def from_connection_uri(
            cls,
            connection_uri_writer: str,
            connection_uri_reader: str,
            pool_size: int = 0,
            max_workers: int = 0
    ) -> ClassVar:
        """Create new object based on connection uri
        :param connection_uri: string connection uri
        :return: object class
        """
        return cls(GraphDbConnection.from_uri(connection_uri_writer, connection_uri_reader, pool_size, max_workers))

    @classmethod
    def from_connection_class(
            cls,
            connection_class: GraphDbConnection
    ) -> ClassVar:
        """Define new class based on object connection
        :param connection_class: object connection class
        :return: object class
        """
        return cls(connection_class)

    @staticmethod
    def prepare_similarity_based_on_homepage_id(content_label, content_homepage_id, graph):
        list_dict_content_similarities = None
        try:
            list_homepage_network = QueryUtils.get_contents_based_on_homepage_id(content_homepage_id, graph)

            list_dataframe_homepage = SimilarityUtils.cluster_data_to_df(list_homepage_network)

            list_new_df_homepage = SimilarityUtils.generate_new_features(list_dataframe_homepage, graph, content_label,
                                                                         content_homepage_id)

            list_tfidf_df = SimilarityUtils.generate_tfidf_matrix(list_new_df_homepage)

            list_dict_content_similarities = SimilarityUtils.calculate_cosine_similarity(list_tfidf_df)

        except Exception as e:
            print(e)
            print('Not able to add content similarity based on home page id')
        return list_dict_content_similarities
