from graphdb import GraphDb
from graphdb.schema import Node
from offline_results.common.constants import LABEL, PROPERTIES, \
    USER_LABEL, CUSTOMER_ID
from offline_results.common.read_write_from_s3 import ConnectS3
from pandas import DataFrame
from datetime import datetime
from offline_results.updater.demographics import PreprocessDemography
from offline_results.common.config import STREAMING_VDB_KEY


class NodeUpdater:

    def __init__(self, graph_connection):
        self.graph = GraphDb.from_connection(graph_connection)

    def process_s3_data(
            self,
            bucket_name=None,
            object_name=None,
            resource=None
    ) -> DataFrame:
        """
        Function to read current month file from s3
        """
        current_month = datetime.now().strftime("%B")
        user_profile = ConnectS3().read_csv_from_s3(bucket_name=bucket_name,
                                                    object_name=STREAMING_VDB_KEY + current_month + '/customer.csv',
                                                    resource=resource)
        user_profile = user_profile.drop(columns=user_profile.columns[0:3].tolist())
        user_profile = PreprocessDemography().controller(df=user_profile)

        return user_profile

    def fetch_user_node(
            self,
            user_id
    ):
        node = Node(
            **{
                LABEL: USER_LABEL,
                PROPERTIES: {CUSTOMER_ID: user_id}
            }
        )
        node_in_graph = self.graph.find_node(node)

        return node_in_graph[0]

    def update_user_node(
            self,
            node,
            properties: dict
    ):
        """
        Function to update user node in graph
        :param node: User Node object
        :param properties: properties to be updated
        """
        self.graph.update_node_property(node, update_query=properties)
        print("Updating done...")

    def controller(
            self,
            bucket_name=None,
            object_name=None,
            resource=None
    ):
        """
        Controller function for the class NodeUpdater
        :param bucket_name: s3 bucket name
        :param object_name: s3 path
        :param resource: boto connection resource
        """
        user_profile = self.process_s3_data(bucket_name=bucket_name,
                                            object_name=object_name,
                                            resource=resource)
        for index in user_profile.to_dict('records'):
            print("Updating node for user {}...".format(index[CUSTOMER_ID]))
            node = self.fetch_user_node(user_id=index[CUSTOMER_ID])
            index.pop(CUSTOMER_ID)
            self.update_user_node(node, index)
